@@ -21,7 +21,7 @@ use Getopt::Long;
     });
     GPD::Profile::Any->set_decoder_options({
         fieldtable  => 1,
-        transform   => $Google::ProtocolBuffers::Dynamic::Fieldtable::profile_decoder_transform,
+        transform   => $Google::ProtocolBuffers::Dynamic::Fieldtable::profile_transform,
     });
 }
 my $sereal_encoder = Sereal::Encoder->new;
@@ -35,7 +35,6 @@ sub main {
         decode_maps         => \&profile_decode_maps,
         decode_objects      => \&profile_decode_objects,
         decode_transform    => \&profile_decode_transform,
-        encode_objects      => \&profile_encode_objects,
     );
 
     my %callgrind_benchmarks = (
@@ -112,7 +111,6 @@ sub random_chars {
 }
 
 sub setup {
-    setup_encode_objects();
     setup_decode_objects();
     setup_decode_maps();
     setup_decode_transform();
@@ -164,7 +162,18 @@ sub profile_decode_maps {
 }
 
 sub setup_decode_objects {
-    my $persons = $benchmarks{encode}{objects};
+    my $make_random_person = sub {
+        my ($id) = @_;
+
+        return {
+            id      => $id,
+            name    => random_chars(3, 16) . ' ' . random_chars(6, 12),
+            email   => random_chars(7, 12) . '@test.com',
+        };
+    };
+    my $persons = {
+        persons => [map $make_random_person->($_), 1 .. 100 ],
+    };
 
     $benchmarks{decode}{objects}{protobuf} = GPD::Profile::PersonArray->encode($persons);
     $benchmarks{decode}{objects}{sereal} = $sereal_encoder->encode($persons);
@@ -239,34 +248,6 @@ sub profile_decode_transform {
     }));
 }
 
-sub setup_encode_objects {
-    my $make_random_person = sub {
-        my ($id) = @_;
-
-        return {
-            id      => $id,
-            name    => random_chars(3, 16) . ' ' . random_chars(6, 12),
-            email   => random_chars(7, 12) . '@test.com',
-        };
-    };
-    my $persons = {
-        persons => [map $make_random_person->($_), 1 .. 100 ],
-    };
-
-    $benchmarks{encode}{objects} = $persons;
-}
-
-sub profile_encode_objects {
-    my ($repeat_count, $which_benchmarks) = @_;
-    my $data = $benchmarks{encode}{objects};
-
-    cmpthese($repeat_count, filter_benchmarks($which_benchmarks, {
-        protobuf_upb    => sub { GPD::Profile::PersonArray->encode($data) },
-        sereal          => sub { $sereal_encoder->encode($data) },
-        json            => sub { $json_decoder->encode($data) },
-    }));
-}
-
 sub start_callgrind {
     system "callgrind_control --instr=on $$";
 }
