@@ -37,10 +37,6 @@ namespace {
     }
 }
 
-void Dynamic::CollectErrors::AddError(const string &filename, int line, int column, const string &message) {
-    croak("Error during protobuf parsing: %s:%d:%d: %s", filename.c_str(), line, column, message.c_str());
-}
-
 MappingOptions::MappingOptions(pTHX_ SV *options_ref) :
         use_bigints(sizeof(IV) < sizeof(int64_t)),
         check_required_fields(true),
@@ -129,10 +125,9 @@ MappingOptions::MappingOptions(pTHX_ SV *options_ref) :
 }
 
 Dynamic::Dynamic(const string &root_directory) :
-        overlay_source_tree(&memory_source_tree, &disk_source_tree),
-        descriptor_loader(&overlay_source_tree, &die_on_error) {
+        descriptor_loader() {
     if (!root_directory.empty())
-        disk_source_tree.MapPath("", root_directory);
+        descriptor_loader.map_disk_path("", root_directory);
 }
 
 Dynamic::~Dynamic() {
@@ -151,6 +146,7 @@ void Dynamic::load_file(pTHX_ const string &file) {
 
     if (loaded)
         add_file_recursively(aTHX_ loaded);
+    descriptor_loader.maybe_croak();
 }
 
 void Dynamic::load_string(pTHX_ const string &file, SV *sv) {
@@ -158,7 +154,7 @@ void Dynamic::load_string(pTHX_ const string &file, SV *sv) {
     const char *data = SvPV(sv, len);
     string actual_file = file.empty() ? "<string>" : file;
 
-    memory_source_tree.AddFile(actual_file, data, len);
+    descriptor_loader.add_memory_file(actual_file, data, len);
     load_file(aTHX_ actual_file);
 }
 
@@ -169,6 +165,7 @@ void Dynamic::load_serialized_string(pTHX_ SV *sv) {
 
     for (vector<const FileDescriptor *>::const_iterator it = loaded.begin(), en = loaded.end(); it != en; ++it)
         add_file_recursively(aTHX_ *it);
+    descriptor_loader.maybe_croak();
 }
 
 void Dynamic::map_wkts(pTHX_ const MappingOptions &options) {
@@ -487,6 +484,7 @@ void Dynamic::mark_package(pTHX_ const string &perl_package, SV *stack_trace) {
 
 void Dynamic::map_message(pTHX_ const Descriptor *descriptor, const string &perl_package, const MappingOptions &options) {
     bool define_perl_names = !options.no_redefine_perl_names || gv_stashpvn(perl_package.data(), perl_package.size(), 0) == NULL;
+
     if (define_perl_names)
         check_package(aTHX_ perl_package, descriptor->full_name());
     if (descriptor_map.find(descriptor->full_name()) != descriptor_map.end())
@@ -630,13 +628,21 @@ void Dynamic::bind_message(pTHX_ const string &perl_package, Mapper *mapper, con
 }
 
 void Dynamic::map_enum(pTHX_ const EnumDescriptor *descriptor, const string &perl_package, const MappingOptions &options) {
-    check_package(aTHX_ perl_package, descriptor->full_name());
+    bool define_perl_names = !options.no_redefine_perl_names || gv_stashpvn(perl_package.data(), perl_package.size(), 0) == NULL;
+
+    if (define_perl_names)
+        check_package(aTHX_ perl_package, descriptor->full_name());
     if (mapped_enums.find(descriptor->full_name()) != mapped_enums.end())
         croak("Enum '%s' has already been mapped", descriptor->full_name().c_str());
 
     mapped_enums.insert(descriptor->full_name());
     mark_package(aTHX_ perl_package, options.stack_trace);
 
+    if (define_perl_names)
+        bind_enum(aTHX_ perl_package, descriptor, options);
+}
+
+void Dynamic::bind_enum(pTHX_ const string &perl_package, const EnumDescriptor *descriptor, const MappingOptions &options) {
     HV *stash = gv_stashpvn(perl_package.data(), perl_package.size(), GV_ADD);
 
     copy_and_bind(aTHX_ "enum_descriptor", perl_package, this, const_cast<EnumDescriptor *>(descriptor));
@@ -653,13 +659,21 @@ void Dynamic::map_service(pTHX_ const ServiceDescriptor *descriptor, const strin
     if (options.client_services == MappingOptions::Disable)
         return;
 
-    check_package(aTHX_ perl_package, descriptor->full_name());
+    bool define_perl_names = !options.no_redefine_perl_names || gv_stashpvn(perl_package.data(), perl_package.size(), 0) == NULL;
+
+    if (define_perl_names)
+        check_package(aTHX_ perl_package, descriptor->full_name());
     if (mapped_services.find(descriptor->full_name()) != mapped_services.end())
         croak("Service '%s' has already been mapped", descriptor->full_name().c_str());
 
     mapped_services.insert(descriptor->full_name());
     mark_package(aTHX_ perl_package, options.stack_trace);
 
+    if (define_perl_names)
+        bind_service(aTHX_ perl_package, descriptor, options);
+}
+
+void Dynamic::bind_service(pTHX_ const string &perl_package, const ServiceDescriptor *descriptor, const MappingOptions &options) {
     ServiceDef *service_def = new ServiceDef(descriptor->full_name());
 
     switch (options.client_services) {
