@@ -53,6 +53,11 @@ Mapper::DecoderHandlers::DecoderHandlers(pTHX_ const Mapper *mapper) :
     mappers.push_back(mapper);
 }
 
+Mapper::DecoderHandlers::~DecoderHandlers() {
+    if (decoder_transform)
+        decoder_transform->destroy(aTHX);
+}
+
 void Mapper::DecoderHandlers::push_mapper(const Mapper *mapper) {
     mappers.push_back(mapper);
     track_seen_fields = mapper->get_track_seen_fields();
@@ -627,10 +632,10 @@ bool Mapper::DecoderHandlers::on_numeric_bool(DecoderHandlers *cxt, const int *f
 
 bool Mapper::DecoderHandlers::on_json_bool(DecoderHandlers *cxt, const int *field_index, bool val) {
     THX_DECLARE_AND_GET;
-    const Mapper *root_mapper = cxt->mappers[0];
+    const Mapper *mapper = cxt->mappers.back();
 
     cxt->mark_seen(field_index);
-    root_mapper->set_json_bool(cxt->get_target(field_index), val);
+    mapper->set_json_bool(cxt->get_target(field_index), val);
 
     return true;
 }
@@ -728,6 +733,8 @@ Mapper::Mapper(pTHX_ Dynamic *_registry, const MessageDef *_message_def, const g
         gpd_descriptor(_gpd_descriptor),
         decoder_field_data(_gpd_descriptor),
         stash(_stash),
+        json_true(NULL),
+        json_false(NULL),
         decoder_callbacks(aTHX_ this),
         string_sink(&output_buffer) {
     SET_THX_MEMBER;
@@ -754,6 +761,13 @@ Mapper::Mapper(pTHX_ Dynamic *_registry, const MessageDef *_message_def, const g
 
         json_true = gv_fetchpvs("JSON::true", 0, SVt_PVGV);
         json_false = gv_fetchpvs("JSON::false", 0, SVt_PVGV);
+
+        if (json_true == NULL || json_false == NULL) {
+            croak("Unable to get JSON true/false values for %s", message_def->full_name());
+        }
+
+        SvREFCNT_inc(json_true);
+        SvREFCNT_inc(json_false);
     }
 
     // on older Perls it is not fully reliable because the check is performed before
@@ -1024,7 +1038,8 @@ Mapper::~Mapper() {
     for (vector<Field>::iterator it = fields.begin(), en = fields.end(); it != en; ++it) {
         if (it->mapper)
             it->mapper->unref();
-        delete it->decoder_transform;
+        if (it->decoder_transform)
+            it->decoder_transform->destroy(aTHX);
     }
     for (vector<MapperField *>::iterator it = extension_mapper_fields.begin(), en = extension_mapper_fields.end(); it != en; ++it)
         // this will make the mapper ref count to go negative, but it's OK
@@ -1033,6 +1048,8 @@ Mapper::~Mapper() {
     // make sure this only goes away after inner destructors have completed
     refcounted_mortalize(aTHX_ registry);
     SvREFCNT_dec(stash);
+    SvREFCNT_dec(json_true);
+    SvREFCNT_dec(json_false);
 }
 
 const char *Mapper::full_name() const {
@@ -1145,7 +1162,7 @@ static DecoderTransform *make_transform(pTHX_ SV *scalar, bool fieldtable) {
         if (fieldtable)
             croak("Fieldtable transformation function must be written in C");
 
-        return new DecoderTransform(maybe_cv);
+        return new DecoderTransform(SvREFCNT_inc(maybe_cv));
     }
 
     if (SvIOK(scalar)) {
@@ -1166,6 +1183,8 @@ void Mapper::set_decoder_options(HV *options) {
     bool fieldtable = fieldtable_sv ? SvTRUE(fieldtable_sv) : false;
 
     if (transform) {
+        if (decoder_callbacks.decoder_transform)
+            decoder_callbacks.decoder_transform->destroy(aTHX);
         decoder_callbacks.decoder_transform = make_transform(aTHX_ transform, fieldtable);
     }
 
@@ -1203,6 +1222,8 @@ void Mapper::set_decoder_options(HV *options) {
                 croak("Can't apply transformation to field %s", field_name.c_str());
             }
 
+            if (field->decoder_transform)
+                field->decoder_transform->destroy(aTHX);
             field->decoder_transform = make_transform(aTHX_ value, fieldtable);
         }
     }
